/* @(#) $Id: _pymalagamodule.c 1.3 Mon, 01 Sep 2008 17:01:01 +0200 mxp $ */

#include <Python.h>
#include <malaga.h>

#if MIN_LIBMALAGA_VERSION < 6
#error Wrong libmalaga version; at least version 6 is required
#endif

/* This is for Python 2.2 */
#ifndef PyMODINIT_FUNC
#define PyMODINIT_FUNC void
#endif

/**/

static PyObject *MalagaError;

/**/

static PyObject *
malaga_to_python(value_t value, int quote_str)
{
   PyObject *py_val;
   PyObject *py_array; /* list */
   PyObject *py_hash;  /* dictionary */

   switch (get_value_type(value))
   {
      case SYMBOL_SYMBOL:
      {
         py_val = Py_BuildValue(
            "s", (char *) get_symbol_name(value_to_symbol(value)));
         break;
      }
  
      case STRING_SYMBOL:
      {
         string_t from_malaga;

         from_malaga = get_value_string(value);

	 if (quote_str)
	    py_val = PyString_FromFormat("\"%s\"", (char *) from_malaga);
	 else
	    py_val = Py_BuildValue("s", (char *) from_malaga);

         free(from_malaga);
         break;
      }
      
      case LIST_SYMBOL:
      {
         value_t item;
	 PyObject *pytem;

         py_array = PyList_New(0);

         for (item = get_first_item(value);
              item != NULL;
              item = get_next_item(value, item))
         {
	    pytem = malaga_to_python(item, quote_str);
	    PyList_Append(py_array, pytem);
	    Py_DECREF(pytem);
         }

         py_val = py_array;
         break;
      }

      case RECORD_SYMBOL:
      {
         value_t item;
	 PyObject *pytem;

         py_hash = PyDict_New();

         for (item = get_first_item(value);
              item != NULL;
              item = get_next_item(value, item))
         {
            symbol_t attrib_symbol = value_to_symbol(item);
            string_t attrib_name   = get_symbol_name(attrib_symbol);

	    pytem = malaga_to_python(get_attribute(value, attrib_symbol),
				     quote_str);
            PyMapping_SetItemString(py_hash, (char *) attrib_name, pytem);
	    Py_DECREF(pytem);
         }

         py_val = py_hash;
         break;
      }
      
      case NUMBER_SYMBOL:
      {
         py_val = PyFloat_FromDouble(value_to_double(value));
         break;
      }
   } /* end of switch */

   return py_val;
}

/**/

static PyObject *
malaga_init(PyObject *self, PyObject *args)
{
   char *project_file;

   if (!PyArg_ParseTuple(args, "s", &project_file))
      return NULL;

   init_libmalaga(project_file);

   if (malaga_error)
   {
      PyErr_SetString(MalagaError, malaga_error);
      return NULL;
   }

   return Py_BuildValue("i", 1);
}

static PyObject *
malaga_terminate(PyObject *self)
{
   terminate_libmalaga();

   Py_INCREF(Py_None);
   return Py_None;
}

static PyObject *
malaga_get_info(PyObject *self)
{
   return Py_BuildValue("s", get_info());
}

static PyObject *
malaga_set_option(PyObject *self, PyObject *args)
{
   char *option;

   if (!PyArg_ParseTuple(args, "s", &option))
      return NULL;
   set_option(option);

   if (malaga_error)
   {
      PyErr_SetString(MalagaError, malaga_error);
      return NULL;
   }

    Py_INCREF(Py_None);
    return Py_None;
}

static PyObject *
malaga_analyze(PyObject *self, PyObject *args) /* TODO: quote_str, mor/syn */
{
   char *item;
   int  analysis_type;
   value_t result;
   int quote_str = 0;
   PyObject *results = PyList_New(0);

   if (!PyArg_ParseTuple(args, "s|ii", &item, &analysis_type, &quote_str))
   {
      Py_DECREF(results);
      return NULL;
   }
   analyse_item(item, analysis_type);

   if (malaga_error)
   {
      PyErr_SetString(MalagaError, malaga_error);
      Py_DECREF(results);
      return NULL;
   }

   if ((result = first_analysis_result()) != NULL)
   {
      PyObject *pytem;

      pytem = malaga_to_python(result, quote_str);
      PyList_Append(results, pytem);
      Py_DECREF(pytem);

      while ((result = next_analysis_result()) != NULL)
      {
	 pytem = malaga_to_python(result, quote_str);
         PyList_Append(results, pytem);
	 Py_DECREF(pytem);
      }
   }

   return results;
}


/* method table */

static PyMethodDef MalagaMethods[] = {
   {"init", (PyCFunction)malaga_init, METH_VARARGS,
    "Load a grammar and initialize."},
   {"get_info", (PyCFunction)malaga_get_info, METH_NOARGS,
    "Get info about the current grammar."},
   {"set_option", (PyCFunction)malaga_set_option, METH_VARARGS,
    "Set a grammar option."},
   {"analyze", (PyCFunction)malaga_analyze, METH_VARARGS,
    "Analyze an item."},
   {"terminate", (PyCFunction)malaga_terminate, METH_NOARGS,
    "Terminate the grammar"},
   {NULL, NULL, 0, NULL}        /* Sentinel */
};

/* the module's initialization function */

PyMODINIT_FUNC
init_pymalaga(void)
{
   PyObject *m;

   m = Py_InitModule("_pymalaga", MalagaMethods);

   MalagaError = PyErr_NewException("malaga.error", NULL, NULL);
   Py_INCREF(MalagaError);
   PyModule_AddObject(m, "error", MalagaError);

   PyModule_AddIntConstant(m, "MORPHOLOGY", 0);
   PyModule_AddIntConstant(m, "SYNTAX", 1);   
}
